package net.swengineer.codegraph.graph.model;

import net.swengineer.codegraph.finders.Usage;

/**
 * Represents an edge in the Code Graph
 * has a source node, target node, and a type
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphEdge {
    private CodeGraphNode source;
    private CodeGraphNode target;
    private Usage.Type type;

    public CodeGraphEdge(CodeGraphNode source, CodeGraphNode target, Usage.Type type) {
        this.source = source;
        this.target = target;
        this.type = type;
    }

    public CodeGraphEdge(Usage usage) {
        this.source = new CodeGraphNode(usage.getSource());
        this.target = new CodeGraphNode(usage.getTarget());
        this.type = usage.getType();
    }

    public CodeGraphNode getSource() {
        return source;
    }

    public CodeGraphNode getTarget() {
        return target;
    }

    public Usage.Type getType() {
        return type;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null || obj.getClass() != this.getClass()) {
            return false;
        }
        CodeGraphEdge other = (CodeGraphEdge) obj;
        return this.source.equals(other.source) &&
                this.target.equals(other.target) &&
                this.type == other.type;
    }

    @Override
    public String toString() {
        return source.toString() + " --" + type.toString() + "-> " + target.toString();
    }

    @Override
    public int hashCode() {
        return type.hashCode() + (source.hashCode() ^ target.hashCode());
    }
}
