package net.swengineer.codegraph.graph.model;

import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiField;
import com.intellij.psi.PsiMethod;
import com.intellij.psi.PsiNamedElement;

/**
 * Represents a node in the Code Graph,
 * a Node could be a Class, Interface, Method or Field
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphNode {
    public static enum Type {
        CLASS, INTERFACE, METHOD, FIELD
    }

    private PsiNamedElement element;
    private Type type;

    public CodeGraphNode(PsiNamedElement element) {
        this.element = element;
        this.type = calculateType(element);
    }

    private Type calculateType(PsiNamedElement element) {
        Type type = null;
        if (element instanceof PsiClass) {
            PsiClass psiClass = (PsiClass) element;
            if (psiClass.isInterface()) {
                type = Type.INTERFACE;
            } else {
                type = Type.CLASS;
            }
        } else if (element instanceof PsiMethod) {
            type = Type.METHOD;
        } else if (element instanceof PsiField) {
            type = Type.FIELD;
        }
        return type;
    }

    public PsiNamedElement getElement() {
        return element;
    }

    public Type getType() {
        return type;
    }

    public String getName() {
        String name = element.getName();
        if (name == null) {
            name = "<<anonymous>>";
        }
        return name;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null || obj.getClass() != this.getClass()) {
            return false;
        }
        CodeGraphNode other = (CodeGraphNode) obj;
        return this.type == other.type &&
                this.element.equals(other.element);
    }

    public boolean isClassNode() {
        return element instanceof PsiClass;
    }

    public boolean isInterfaceNode() {
        return isClassNode() && ((PsiClass) element).isInterface();
    }

    public boolean isMethodNode() {
        return element instanceof PsiMethod;
    }

    public boolean isFieldNode() {
        return element instanceof PsiField;
    }

    @Override
    public String toString() {
        return getName();
    }

    @Override
    public int hashCode() {
        return element.hashCode();
    }
}
