package net.swengineer.codegraph.graph.presentation;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.graph.builder.actions.*;
import com.intellij.openapi.graph.builder.actions.layout.ApplyCurrentLayoutAction;
import com.intellij.openapi.graph.builder.actions.printing.PrintGraphAction;
import com.intellij.openapi.graph.builder.actions.printing.PrintPreviewAction;
import com.intellij.openapi.util.IconLoader;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.presentation.filterer.CodeGraphFiltererDialog;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;

/**
 * a component which holds paper (which holds the graph) and a toolbar
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphComponent {
    private final static Icon ICON = IconLoader.getIcon("/icons/inspector/useFilter.png");
    private final JComponent myComponent;
    private CodeGraph codeGraph;


    public CodeGraphComponent(@NotNull CodeGraph codeGraph) {
        this.codeGraph = codeGraph;
        myComponent = new JPanel(new BorderLayout());

        DefaultActionGroup actions = getToolbarActionGroup();
        ActionToolbar actionToolbar = ActionManager.getInstance().createActionToolbar(ActionPlaces.UNKNOWN, actions, true);
        myComponent.add(actionToolbar.getComponent(), BorderLayout.NORTH);
        myComponent.add(codeGraph.getView().getComponent(), BorderLayout.CENTER);

        codeGraph.initialize();
    }

    private DefaultActionGroup getToolbarActionGroup() {
        DefaultActionGroup actions = new DefaultActionGroup();
        actions.add(new ShowHideGridAction(codeGraph.getGraph()));
        actions.add(new SnapToGridAction(codeGraph.getGraph()));
        actions.addSeparator();

        actions.add(new ZoomInAction(codeGraph.getGraph()));
        actions.add(new ZoomOutAction(codeGraph.getGraph()));

        actions.add(new ActualZoomAction(codeGraph.getGraph()));

        actions.add(new FitContentAction(codeGraph.getGraph()));
        actions.addSeparator();

        actions.add(new ApplyCurrentLayoutAction(codeGraph.getGraph()));
        actions.addSeparator();


        actions.add(new AnAction(null, null, ICON) {
            public void actionPerformed(AnActionEvent e) {
                new CodeGraphFiltererDialog(codeGraph).show();
            }
        });
        actions.addSeparator();

        actions.add(new DeleteSelectionAction());
        actions.addSeparator();

        actions.add(new PrintGraphAction(codeGraph.getGraph()));
        actions.add(new PrintPreviewAction(codeGraph.getGraph()));
        actions.addSeparator();

        return actions;
    }

    @NotNull
    public JComponent getComponent() {
        return myComponent;
    }

}
