package net.swengineer.codegraph.graph.presentation;

import com.intellij.openapi.graph.GraphManager;
import com.intellij.openapi.graph.view.Graph2DNodeRealizer;
import com.intellij.openapi.graph.view.NodeLabel;
import com.intellij.openapi.graph.view.NodeRealizer;
import com.intellij.openapi.graph.view.ShapeNodeRealizer;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import net.swengineer.codegraph.plugin.util.CodeGraphLogger;

import java.awt.*;

/**
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphRealizerFactory {
    public static NodeRealizer createDefaultNodeRealizer(CodeGraphNode codeGraphNode) {
        long start = System.currentTimeMillis();

        Graph2DNodeRealizer nodeRealizer = GraphManager.getGraphManager().createGraph2DNodeRealizer();

        String typeText = CodeGraphTypeNames.getTypeName(codeGraphNode.getType());
        String nameText = codeGraphNode.getName();

        NodeLabel typeLabel = addNodeTypeLabel(typeText, nodeRealizer);
        NodeLabel nameLabel = addNodeNameLabel(nameText, nodeRealizer);

        calculateAndSetNodeSize(nodeRealizer, typeLabel, nameLabel);

        setNodeColor(nodeRealizer, codeGraphNode);

        setNodeShape(nodeRealizer);

        long time = System.currentTimeMillis() - start;
        CodeGraphLogger.debug("Building NodeRealizer for : " + codeGraphNode.toString() + " (took " + time + "ms)");

        return nodeRealizer;
    }

    private static void setNodeShape(Graph2DNodeRealizer nodeRealizer) {
        nodeRealizer.setShapeType(ShapeNodeRealizer.ROUND_RECT);
    }

    private static void setNodeColor(NodeRealizer nodeRealizer, CodeGraphNode codeGraphNode) {
        Color color = Color.WHITE;

        CodeGraphNode.Type type = codeGraphNode.getType();
        switch (type) {
            case CLASS:
                color = new Color(76, 131, 255);
                break;
            case INTERFACE:
                color = new Color(76, 255, 131);
                break;
            case METHOD:
                color = new Color(255, 131, 76);
                break;
            case FIELD:
                color = new Color(219, 255, 76);
                break;
        }

        nodeRealizer.setFillColor(color);
    }

    private static void calculateAndSetNodeSize(NodeRealizer nodeRealizer, NodeLabel typeLabel, NodeLabel nameLabel) {
        double width = 5 + Math.max(typeLabel.getWidth(), nameLabel.getWidth()) + 5;
        double height = 3 + typeLabel.getHeight() + 1 + nameLabel.getHeight() + 3;
        nodeRealizer.setSize(width, height);
    }

    private static NodeLabel addNodeNameLabel(String nameText, NodeRealizer nodeRealizer) {
        NodeLabel nameLabel = GraphManager.getGraphManager().createNodeLabel();
        nameLabel.setText(nameText);
        nameLabel.setFontSize(14);
        nameLabel.setModel(NodeLabel.INTERNAL);
        nameLabel.setPosition(NodeLabel.BOTTOM);
        nodeRealizer.addLabel(nameLabel);
        return nameLabel;
    }

    private static NodeLabel addNodeTypeLabel(String typeText, NodeRealizer nodeRealizer) {
        NodeLabel typeLabel = GraphManager.getGraphManager().createNodeLabel();
        typeLabel.setText("<<" + typeText + ">>");
        typeLabel.setFontSize(10);
        typeLabel.setModel(NodeLabel.INTERNAL);
        typeLabel.setPosition(NodeLabel.TOP);
        nodeRealizer.addLabel(typeLabel);
        return typeLabel;
    }
}
