package net.swengineer.codegraph.graph.presentation.filterer;

import com.intellij.CommonBundle;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.ui.DialogWrapper;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * a dialog for choosing node/edge types to filter out of the graph
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphFiltererDialog extends DialogWrapper {
    private CodeGraphFiltererPanel filtererPanel;
    private CodeGraph codeGraph;

    public CodeGraphFiltererDialog(CodeGraph codeGraph) {
        super(false);
        this.codeGraph = codeGraph;
        filtererPanel = new CodeGraphFiltererPanel();
        CodeGraphNode.Type[] nodeTypes = CodeGraphNode.Type.values();
        for (CodeGraphNode.Type nodeType : nodeTypes) {
            boolean filtered = codeGraph.getGraphFilterer().shouldFilter(nodeType);
            filtererPanel.setSelected(nodeType, !filtered);
        }

        Usage.Type[] edgeTypes = Usage.Type.values();
        for (Usage.Type edgeType : edgeTypes) {
            boolean filtered = codeGraph.getGraphFilterer().shouldFilter(edgeType);
            filtererPanel.setSelected(edgeType, !filtered);
        }

        setTitle("Select visible types");

        init();
    }

    @Nullable
    protected JComponent createCenterPanel() {
        return filtererPanel;
    }

    @Override
    protected Action[] createActions() {
        return new Action[]{
                createOKAction(),
                getCancelAction()
        };
    }

    protected Action createOKAction() {
        return new AbstractAction(CommonBundle.getOkButtonText()) {
            public void actionPerformed(ActionEvent e) {

                ProgressManager.getInstance().runProcessWithProgressSynchronously(
                        new Runnable() {
                            public void run() {
                                codeGraph.getGraphFilterer().clear();
                                CodeGraphNode.Type[] nodeTypes = CodeGraphNode.Type.values();
                                for (CodeGraphNode.Type nodeType : nodeTypes) {
                                    codeGraph.getGraphFilterer().setShouldFilter(nodeType, !filtererPanel.isSelected(nodeType));
                                }

                                Usage.Type[] edgeTypes = Usage.Type.values();
                                for (Usage.Type edgeType : edgeTypes) {
                                    codeGraph.getGraphFilterer().setShouldFilter(edgeType, !filtererPanel.isSelected(edgeType));
                                }
                                codeGraph.updateGraph();
                            }
                        },
                        "Applying filter",
                        false,
                        codeGraph.getProject());
                dispose();
            }
        };
    }
}
