package net.swengineer.codegraph.graph.presentation.filterer;

import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import net.swengineer.codegraph.graph.presentation.CodeGraphTypeNames;

import javax.swing.*;
import java.util.List;

/**
 * This panel holds checkboxes for all node and edges type
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphFiltererPanel extends JPanel {
    private final static String NODE_TYPES = "Node Types";
    private final static String EDGE_TYPES = "Edge Types";

    private EnumCheckboxesPanel<CodeGraphNode.Type> nodeTypesPanel;
    private EnumCheckboxesPanel<Usage.Type> edgeTypesPanel;

    public CodeGraphFiltererPanel() {
        createTypesPanels();
        buildUI();
    }

    private void buildUI() {
        Box box = Box.createHorizontalBox();
        box.add(nodeTypesPanel);
        box.add(edgeTypesPanel);
        add(box);
    }

    private void createTypesPanels() {
        nodeTypesPanel = new EnumCheckboxesPanel<CodeGraphNode.Type>(CodeGraphNode.Type.class, NODE_TYPES, true) {
            public String getDisplayName(CodeGraphNode.Type type) {
                return CodeGraphTypeNames.getTypeName(type);
            }
        };
        edgeTypesPanel = new EnumCheckboxesPanel<Usage.Type>(Usage.Type.class, EDGE_TYPES, true) {
            public String getDisplayName(Usage.Type type) {
                return CodeGraphTypeNames.getTypeName(type);
            }
        };
    }

    public List<CodeGraphNode.Type> getSelectedNodeTypes() {
        return nodeTypesPanel.getSelected();
    }

    public List<Usage.Type> getSelectedEdgeTypes() {
        return edgeTypesPanel.getSelected();
    }

    public boolean isSelected(CodeGraphNode.Type nodeType) {
        return nodeTypesPanel.isSelected(nodeType);
    }

    public boolean isSelected(Usage.Type edgeType) {
        return edgeTypesPanel.isSelected(edgeType);
    }

    public void setSelected(CodeGraphNode.Type nodeType, boolean selected) {
        nodeTypesPanel.setSelected(nodeType, selected);
    }

    public void setSelected(Usage.Type edgeType, boolean selected) {
        edgeTypesPanel.setSelected(edgeType, selected);
    }
}
