package net.swengineer.codegraph.graph.presentation.filterer;

import javax.swing.*;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

/**
 * This abstract panel is a panel which contained checkboxes for all
 * values of an enum
 * Extending class should implement the method getDisplayName in order
 * to provide naming for the enum values;
 *
 * @author Aviram Segal
 * @version 1.0
 */
public abstract class EnumCheckboxesPanel<E extends Enum<E>> extends JPanel {
    List<E> enums;
    List<JCheckBox> checkBoxes;

    protected EnumCheckboxesPanel(Class<E> en) {
        this(en, false);
    }

    protected EnumCheckboxesPanel(Class<E> en, boolean selected) {
        this(en, "", selected);
    }

    protected EnumCheckboxesPanel(Class<E> en, String title, boolean selected) {
        enums = new ArrayList<E>(EnumSet.allOf(en));
        createCheckboxes(selected);
        addCheckBoxesToPanel(title);
    }

    private void addCheckBoxesToPanel(String title) {
        Box box = Box.createVerticalBox();
        box.add(new JLabel(title));
        for (JCheckBox checkBox : checkBoxes) {
            box.add(checkBox);
        }
        add(box);
    }

    private void createCheckboxes(boolean selected) {
        checkBoxes = new ArrayList<JCheckBox>();
        for (E e : enums) {
            JCheckBox checkBox = new JCheckBox(getDisplayName(e), selected);
            checkBoxes.add(checkBox);
        }
    }

    public abstract String getDisplayName(E e);

    public List<E> getSelected() {
        List<E> selected = new ArrayList<E>();
        int size = checkBoxes.size();
        assert size == enums.size();
        for (int i = 0; i < size; i++) {
            JCheckBox checkBox = checkBoxes.get(i);
            if (checkBox.isSelected()) {
                selected.add(enums.get(i));
            }
        }
        return selected;
    }

    public boolean isSelected(E e) {
        return checkBoxes.get(enums.indexOf(e)).isSelected();
    }

    public void setSelected(E e, boolean selected) {
        checkBoxes.get(enums.indexOf(e)).setSelected(selected);
    }
}
