package net.swengineer.codegraph.graph.presentation.popupmenu;

import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.graph.view.PopupMode;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;

/**
 * a custom popup menu for CodeGraph
 * holds the default PopupMode and customizes it
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphPopupMode extends PopupMode {
    CodeGraph codeGraph;
    @Nullable
    PopupMode defaultPopupMode;

    public CodeGraphPopupMode(CodeGraph codeGraph, PopupMode defaultPopupMode) {
        this.codeGraph = codeGraph;
        this.defaultPopupMode = defaultPopupMode;
    }

    @Override
    public JPopupMenu getNodePopup(Node v) {
        JPopupMenu menu;
        CodeGraphNode node = codeGraph.getCodeGraphNode(v);
        if (node.isClassNode()) {
            menu = getClassPopupMenu(v);
        } else if (node.isInterfaceNode()) {
            menu = getInterfacePopupMenu(v);
        } else if (node.isMethodNode()) {
            menu = getMethodPopupMenu(v);
        } else if (node.isFieldNode()) {
            menu = getFieldPopupMenu(v);
        } else {
            menu = new JPopupMenu();
        }
        return menu;
    }

    @Override
    public JPopupMenu getSelectionPopup(double x, double y) {
        if (defaultPopupMode != null) {
            return defaultPopupMode.getSelectionPopup(x, y);
        } else {
            return null;
        }
    }

    @Override
    public JPopupMenu getPaperPopup(double x, double y) {
        if (defaultPopupMode != null) {
            return defaultPopupMode.getPaperPopup(x, y);
        } else {
            return null;
        }
    }

    private JPopupMenu getClassPopupMenu(Node v) {
        JPopupMenu menu = getCommonNodePopupMenu(v);
        menu.addSeparator();
        ClassMenuActions.addAllActions(codeGraph, menu, v);
        return menu;
    }

    private JPopupMenu getInterfacePopupMenu(Node v) {
        return getClassPopupMenu(v);
    }

    private JPopupMenu getMethodPopupMenu(Node v) {
        JPopupMenu menu = getCommonNodePopupMenu(v);
        menu.addSeparator();
        MethodMenuActions.addAllActions(codeGraph, menu, v);
        return menu;
    }

    private JPopupMenu getFieldPopupMenu(Node v) {
        JPopupMenu menu = getCommonNodePopupMenu(v);
        menu.addSeparator();
        FieldMenuActions.addAllActions(codeGraph, menu, v);
        return menu;
    }

    private JPopupMenu getCommonNodePopupMenu(final Node v) {
        JPopupMenu menu = new JPopupMenu();
        CommonMenuActions.addAllActions(codeGraph, menu, v);
        return menu;
    }

/*    private void removeNode(Node v) {
        codeGraph.removeNode(v);
    }*/

}
