package net.swengineer.codegraph.graph.presentation.popupmenu;

import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiField;
import net.swengineer.codegraph.finders.FieldUsageFinder;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.graph.model.CodeGraphNode;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Set;

/**
 * Menu actions for a field's popup menu
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class FieldMenuActions {
    public static void addAllActions(CodeGraph codeGraph, JPopupMenu menu, Node node) {
        menu.add(getAddContainingClassAction(codeGraph, node));
        menu.add(getFieldTypeAction(codeGraph, node));
        menu.add(getUsingMethodsAction(codeGraph, node));
    }

    private static Action getUsingMethodsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add using methods") {
            public void actionPerformed(ActionEvent e) {
                new FieldUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, FieldUsageFinder finder) {
                        return finder.getUsingMethodsUsages();
                    }
                }.run(node);
            }
        };
    }

    private static Action getFieldTypeAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add field's type") {
            public void actionPerformed(ActionEvent e) {
                new FieldUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, FieldUsageFinder finder) {
                        return finder.getFieldTypeUsage();
                    }
                }.run(node);
            }
        };
    }

    private static Action getAddContainingClassAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add containing class") {
            public void actionPerformed(ActionEvent e) {
                new FieldUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, FieldUsageFinder finder) {
                        return finder.getContainingClassUsage();
                    }
                }.run(node);
            }
        };
    }

    private abstract static class FieldUsageAdder {
        private CodeGraph codeGraph;

        public FieldUsageAdder(CodeGraph codeGraph) {
            this.codeGraph = codeGraph;
        }

        protected abstract Set<Usage> getUsages(CodeGraph codeGraph, FieldUsageFinder finder);

        public void run(Node node) {
            Project project = codeGraph.getProject();
            CodeGraphNode cgNode = codeGraph.getCodeGraphNode(node);
            PsiField psiField = (PsiField) cgNode.getElement();
            FieldUsageFinder finder = FieldUsageFinder.create(psiField, project.getProjectScope());

            Set<Usage> usages = getUsages(codeGraph, finder);
            for (Usage usage : usages) {
                codeGraph.getDataModel().addEdge(new CodeGraphEdge(usage));
            }

            codeGraph.updateGraph();
            codeGraph.fitContent();
        }
    }
}
