package net.swengineer.codegraph.plugin.project;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.Key;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.ui.content.Content;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.presentation.CodeGraphComponent;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;

/**
 * The tool window holding the graph component
 *
 * @author Aviram Segal
 * @version 1.0
 * @see {CodeGraphComponent}
 */
public class CodeGraphToolWindow implements ProjectComponent {
    private Project project;
    private CodeGraph codeGraph;

    public static final String TOOL_WINDOW_ID = "Code Graph";
    public static final Key<CodeGraph> CODE_GRAPH_KEY = Key.create("Code Graph");

    public CodeGraph getCodeGraph() {
        return codeGraph;
    }

    public Project getProject() {
        return project;
    }

    public CodeGraphToolWindow(Project project) {
        this.project = project;

    }

    public void initComponent() {

    }

    public void disposeComponent() {

    }

    @NotNull
    public String getComponentName() {
        return "net.swengineer.codegraph.plugin.project.CodeGraphToolWindow";
    }

    public void projectOpened() {
        createToolWindow();
    }

    public void projectClosed() {
        destroyToolWindow();
    }

    private void createToolWindow() {
        ToolWindowManager toolWindowManager;
        toolWindowManager = ToolWindowManager.getInstance(project);

        toolWindowManager.registerToolWindow(TOOL_WINDOW_ID, false, ToolWindowAnchor.RIGHT);

        codeGraph = CodeGraph.createGraph(project);
        project.putUserData(CODE_GRAPH_KEY, codeGraph);

        codeGraph.initialize();
        ToolWindow toolWindow = toolWindowManager.getToolWindow(CodeGraphToolWindow.TOOL_WINDOW_ID);
        JComponent component = new CodeGraphComponent(codeGraph).getComponent();
        Content content = toolWindow.getContentManager().getFactory().createContent(component, "", false);
        toolWindow.getContentManager().addContent(content);
        toolWindow.activate(null);
    }

    private void destroyToolWindow() {
        ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(project);
        ToolWindow toolWindow = toolWindowManager.getToolWindow(CodeGraphToolWindow.TOOL_WINDOW_ID);
        toolWindow.getContentManager().removeAllContents(true);
        toolWindowManager.unregisterToolWindow(TOOL_WINDOW_ID);
    }
}
